﻿#region

using MVR.FileManagementSecure;
using SimpleJSON;

#endregion

namespace SceneNinja {
    public class PlayerData {

        public readonly string ID;

        private readonly string _label;
        private bool RightSide { get; }
        public string AppearancePath { get; private set; }
        public string PluginsPath { get; private set; }
        public bool SuppressClothing { get; private set; }
        public bool SuppressScale { get; private set; }
        public bool MergePlugins { get; private set; }
        public bool IgnorePluginVersion { get; private set; }


        private JSONStorableBool _suppressClothing;
        private JSONStorableBool _suppressScale;
        private JSONStorableBool _mergePlugins;
        private JSONStorableBool _ignorePluginVersion;

        private JSONStorableString _appearancePath;
        private JSONStorableString _pluginsPath;

        private JSONStorableString _appearanceText;
        private JSONStorableString _pluginsText;

        private SceneNinja _ninja;

        public PlayerData(string id) {
            ID = id;
            _suppressClothing = new JSONStorableBool(ID + "SuppressClothing", true, BoolUpdated);
            _suppressScale = new JSONStorableBool(ID + "SuppressScale", true, BoolUpdated);
            _mergePlugins = new JSONStorableBool(ID + "MergePlugins", true, BoolUpdated);
            _ignorePluginVersion = new JSONStorableBool(ID + "IgnorePluginVersion", true, BoolUpdated);
            _appearancePath = new JSONStorableString(ID + "ApperancePath", "", StringUpdated);
            _pluginsPath = new JSONStorableString(ID + "PluginsPath", "", StringUpdated);
            RightSide = id.Contains("M");
            AppearancePath = "";
            PluginsPath = "";
            _appearanceText = new JSONStorableString("AppearanceText", "");
            _pluginsText = new JSONStorableString("PluginsText", "");
            SuppressClothing = true;
            SuppressScale = true;
            if (ID.Contains("M")) {
                _label = ID == "M1" ? "Primary Male" : "Secondary Male(s)";
            } else {
                _label = ID == "F1" ? "Primary Female" : "Secondary Female(s)";
            }
            RightSide = ID.Contains("M");
        }
        public PlayerData(string oId, JSONNode obj) {
            ID = oId;
            _label = obj["label"];
            SuppressClothing = obj["suppressClothing"].AsBool;
            SuppressScale = obj["suppressScale"].AsBool;
            MergePlugins = obj["mergePlugins"].AsBool;
            IgnorePluginVersion = obj["ignorePluginVersion"].AsBool;
            AppearancePath = obj["appearancePath"];
            PluginsPath = obj["pluginsPath"];
            if (ID.Contains("M")) {
                _label = ID == "M1" ? "Primary Male" : "Secondary Male(s)";
            } else {
                _label = ID == "F1" ? "Primary Female" : "Secondary Female(s)";
            }
            RightSide = ID.Contains("M");
        }

        public void LoadVars() {
            _suppressClothing = new JSONStorableBool(ID + "SuppressClothing", SuppressClothing, BoolUpdated);
            _suppressScale = new JSONStorableBool(ID + "SuppressScale", SuppressScale, BoolUpdated);
            _mergePlugins = new JSONStorableBool(ID + "MergePlugins", MergePlugins, BoolUpdated);
            _ignorePluginVersion = new JSONStorableBool(ID + "IgnorePluginVersion", IgnorePluginVersion, BoolUpdated);
            _appearancePath = new JSONStorableString(ID + "ApperancePath", AppearancePath, StringUpdated);
            _pluginsPath = new JSONStorableString(ID + "PluginsPath", PluginsPath, StringUpdated);
            _appearanceText = new JSONStorableString(ID + "ApperanceText", "");
            _pluginsText = new JSONStorableString(ID + "PluginsText", "");


            if (AppearancePath != "") {
                var fileName = AppearancePath;
                var parts = fileName.Split('/');
                var file = parts[parts.Length - 1];
                _appearanceText.val = file;
            }

            if (PluginsPath != "") {
                var fileName = PluginsPath;
                var parts = fileName.Split('/');
                var file = parts[parts.Length - 1];
                _pluginsText.val = file;
            }
        }


        private void StringUpdated(JSONStorableString js) {
            var name = js.name;
            SceneNinja.LogMessage("String updated " + name);
            if (name.Contains("Appearance")) {
                AppearancePath = js.val;
            } else {
                PluginsPath = js.val;
            }

            _ninja.UpdatePlayer(this);
        }

        private void BoolUpdated(JSONStorableBool jb) {
            var name = jb.name;
            if (name.Contains("Clothing")) {
                SuppressClothing = jb.val;
            }

            if (name.Contains("Scale")) {
                SuppressScale = jb.val;
            }

            if (name.Contains("Merge")) {
                MergePlugins = jb.val;
            }

            if (name.Contains("Ignore")) {
                IgnorePluginVersion = jb.val;
            }

            _ninja.UpdatePlayer(this);
        }

        public void Initialize(SceneNinja ninja) {
            _ninja = ninja;
            ninja.RegisterBool(_suppressClothing);
            ninja.RegisterBool(_suppressScale);
            ninja.RegisterBool(_mergePlugins);
            ninja.RegisterBool(_ignorePluginVersion);
            ninja.RegisterString(_appearancePath);
            ninja.RegisterString(_pluginsPath);
            ninja.CreateUILabel(_label, RightSide);
            ninja.CreateUIButton(ID + " Appearance", RightSide, SelectAppearanceFile);
            ninja.CreateUIButton("Clear Appearance", RightSide, ClearAppearanceFile);
            ninja.CreateUITextField(_appearanceText, RightSide, 40);
            ninja.CreateUIToggle(_suppressClothing, RightSide, "Suppress Clothing Load");
            ninja.CreateUIToggle(_suppressScale, RightSide, "Suppress Scale Load");
            ninja.CreateUIButton(ID + " Plugins", RightSide, SelectPluginPreset);
            ninja.CreateUIButton("Clear Plugins", RightSide, ClearPluginsFile);
            ninja.CreateUITextField(_pluginsText, RightSide, 40);
            ninja.CreateUIToggle(_mergePlugins, RightSide, "Merge Load Plugins");
            ninja.CreateUIToggle(_ignorePluginVersion, RightSide, "Ignore Plugin Versions");
        }

        private void ClearPluginsFile() {
            PluginsPath = "";
            _pluginsPath.val = "";
            _pluginsText.val = "";
            _ninja.UpdatePlayer(this);
        }

        private void ClearAppearanceFile() {
            AppearancePath = "";
            _appearancePath.val = "";
            _appearanceText.val = "";
            _ninja.UpdatePlayer(this);
        }

        private void SelectPluginPreset() {
            var shortcuts =
                FileManagerSecure.GetShortCutsForDirectory("Custom\\Atom\\Person\\Plugins", false, false, true);
            SuperController.singleton.GetMediaPathDialog(PluginFileSelected, "vap", "Custom\\Atom\\Person\\Plugins",
                false,
                true, false, null, false, shortcuts);
        }

        private void SelectAppearanceFile() {
            var shortcuts =
                FileManagerSecure.GetShortCutsForDirectory("Custom\\Atom\\Person\\Appearance", false, false, true);
            SuperController.singleton.GetMediaPathDialog(AppearanceFileSelected, "vap",
                "Custom\\Atom\\Person\\Appearance", false, true, false, null, false, shortcuts);
        }

        private void PluginFileSelected(string fileName) {
            var pFile = "";
            if (fileName != "" && !fileName.Contains("Appearance")) {
                SceneNinja.LogMessage("Plugins selected: " + fileName);
                PluginsPath = fileName;
                var parts = fileName.Split('/');
                pFile = parts[parts.Length - 1];
            }

            _pluginsPath.val = fileName;
            _pluginsText.val = pFile;
        }

        private void AppearanceFileSelected(string fileName) {
            var aFile = "";
            if (fileName != "" && !fileName.Contains("Plugins")) {
                SceneNinja.LogMessage("Appearance selected: " + fileName);
                AppearancePath = fileName;
                var parts = fileName.Split('/');
                aFile = parts[parts.Length - 1];
            }

            _appearancePath.val = fileName;
            _appearanceText.val = aFile;
        }

        public JSONNode ToJson() {
            var data = new JSONClass {
                { "id", ID },
                { "label", _label },
                { "appearancePath", AppearancePath },
                { "pluginsPath", PluginsPath },
                { "suppressClothing", SuppressClothing.ToString() },
                { "suppressScale", SuppressScale.ToString() },
                { "ignorePluginVersion", IgnorePluginVersion.ToString() },
                { "mergePlugins", MergePlugins.ToString() }
            };
            return data;
        }
    }
}
